@@ -10,3 +10,9 @@
 #undef HAVE_WORDEXP
 #undef HAVE_MPFR
 #undef GETTEXT_PACKAGE
+#undef HAVE_GTKSOURCEVIEW
+#undef USE_NCURSES
+#undef INC_NCURSES
+#undef HAVE_SETENV
+#undef HAVE_UNSETENV
+#undef HAVE_CLEARENV
